# using concerns (it's a rails thing)
module Voteable
  extend ActiveSupport::Concern

  included do
    has_many :votes, as: :voteable
  end

  def diff_votes
    self.up_votes - self.down_votes
  end

  def up_votes
    self.votes.where(vote: true).size
  end

  def down_votes
    self.votes.where(vote: false).size
  end
end

# normal metaprogramming
# module Voteable
#   def self.included(base)
#     base.send(:include, InstanceMethods)
#     base.extend ClassMethods
#     base.class_eval do 
#       associate_votes
#     end
#   end

#   module InstanceMethods
#     def diff_votes
#       self.up_votes - self.down_votes
#     end

#     def up_votes
#       self.votes.where(vote: true).size
#     end

#     def down_votes
#       self.votes.where(vote: false).size
#     end
#   end

#   module ClassMethods
#     def associate_votes
#       has_many :votes, as: :voteable
#     end
#   end
# end